/******************************************************************************
 *                                                                            *
 * File:   exectree.c         Version  1.01            Date: 1996-08-03       *
 *                                                                            *
 * Copyright (C) 1993-1996 by kostis@acm.prg (Kosta Kostis)                   *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *        - walk through a directory tree calling a program once in every dir *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     1996-08-03: KK V1.01                                                   *
 *        - added a few comments                                              *
 *     1993-04-28: KK V1.00                                                   *
 *        - initial coding                                                    *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <dir.h>
#include <dos.h>

#include "exectree.h"

/******************************************************************************

	global variables for Ctrl-Break processing

 *****************************************************************************/

static	char	CurDir		[MAX_PATHNAME + 1] ;
static	int	CurDrive	= -1 ;

/******************************************************************************

	Function:
		display copyright message

	Parameters:
		none

	Returns:
		nothing

 *****************************************************************************/

static	void	Hello (void)
{
	fprintf (stderr, MSG_COPYRIGHT) ;
	fprintf (stderr, MSG_VERSION, PRGNAM, VERSION) ;
}

/******************************************************************************

	Function:
		return to initial directory if we're interrupted

	Parameters:
		none

	Returns:
		nothing

 *****************************************************************************/

void    CtrlBreak
(
	int		Sig
)
{
	if (Sig != SIGINT)
		return ;

	if (CurDrive != -1)
	{
		setdisk (CurDrive) ;
		chdir (CurDir) ;
	}

	exit (ERR_BREAK) ;
}

/******************************************************************************

	Function:
		add file mask for all files and open directory

	Parameters:
		struct  ffblk   *Tree   pointer through directory tree
		char    *CurDir         current directory

	Returns:
		0                       if dir can be opened successfully

 *****************************************************************************/

static	int	FindTree
(
	struct	ffblk	*Tree,
	char		*CurDir
)
{
	char		TmpDir		[MAX_PATHNAME + 1] ;

	strcpy (TmpDir, CurDir) ;
	if (TmpDir [strlen (TmpDir) - 1] != '\\')
		strcat (TmpDir, "\\") ;
	strcat (TmpDir, "*.*") ;

	return (findfirst (TmpDir, Tree, FA_DIREC)) ;
}

/******************************************************************************

	Function:
		Walk through directory tree and fire up program once in
		every directory

	Parameters:
		struct  ffblk   *Root   pointer through directory tree
		char    *CurDir         current directory
		char    *Cmd            command to be called

	Returns:
		nothing

 *****************************************************************************/

static	void	ExecTree
(
	struct	ffblk	*Root,
	char		*CurDir,
	char		*Cmd
)
{
	char		NewDir		[MAX_PATHNAME + 1] ;
	struct	ffblk	Tree ;

	/**********************************************************************

		perform action in directory *once*

	 *********************************************************************/

	chdir (CurDir) ;
	fprintf (stderr, MSG_PROCESS, PRGNAM, CurDir) ;
	system (Cmd) ;

	/**********************************************************************

		scan through this directory for subdirectories

	 *********************************************************************/

	do
	{
		if
		(
			(Root -> ff_attrib & FA_DIREC) &&
			(strncmp (Root -> ff_name, ".", 1) != 0)
		)
		{
			/******************************************************

				scan through subdirectory

			 *****************************************************/

			strcpy (NewDir, CurDir) ;
			if (NewDir [strlen (NewDir) - 1] != '\\')
				strcat (NewDir, "\\") ;
			strcat (NewDir, Root -> ff_name) ;
			if (FindTree (&Tree, NewDir) == 0)
				ExecTree (&Tree, NewDir, Cmd) ;
		}
	}
	while (findnext (Root) == 0) ;
}

/*****************************************************************************/

int     main
(
	int		argc,
	char		*argv		[]
)
{
	char		Cmd		[MAX_CMDLINE + 1] ;
	char		NewDir		[MAX_PATHNAME + 1] ;
	int		NewDrive ;
	struct	ffblk	Root ;
	int		i ;

	/**********************************************************************

		check syntax first

	 *********************************************************************/

	if (argc < 3)
	{
		Hello () ;
		fprintf (stderr, ERR_MSG_SYNTAX, PRGNAM) ;
		return (ERR_SYNTAX) ;
	}

	/**********************************************************************

		build command line with all parameters supplied

	 *********************************************************************/

	strcpy (Cmd, argv [2]) ;
	for (i = 3 ; i < argc ; ++i)
	{
		strcat (Cmd, " ") ;
		strcat (Cmd, argv [i]) ;
	}

	/**********************************************************************

		figure out current directory and drive

	 *********************************************************************/

	CurDrive = getdisk () ;
	if (getcwd (CurDir, MAX_PATHNAME) == (char *) NULL)
	{
		fprintf (stderr, ERR_MSG_CURDIR, PRGNAM) ;
		return (ERR_CURDIR) ;
	}

	/**********************************************************************

		prevent changed cwd after Ctrl-Break

	 *********************************************************************/

	signal (SIGINT, CtrlBreak) ;

	/**********************************************************************

		do we have to change the drive?

	 *********************************************************************/

	strupr (argv [1]) ;
	if (argv [1][1] == ':')
	{
		NewDrive = argv [1][0] - 'A' ;
		setdisk (NewDrive) ;
	}

	/**********************************************************************

		change working directory

	 *********************************************************************/

	chdir (argv [1]) ;
	if (getcwd (NewDir, MAX_PATHNAME) == (char *) NULL)
	{
		fprintf (stderr, ERR_MSG_CHDIR, PRGNAM) ;
		return (ERR_CHDIR) ;
	}

	/**********************************************************************

		scan through directories

	 *********************************************************************/

	if (FindTree (&Root, NewDir) == 0)
		ExecTree (&Root, NewDir, Cmd) ;

	/**********************************************************************

		return to the directory and drive where we started

	 *********************************************************************/

	setdisk (CurDrive) ;
	chdir (CurDir) ;

	return (ERR_NONE) ;
}
